@@ -225,7 +225,7 @@ latexmk_emu = function(
   }
   run_engine()
   # some problems only trigger warnings but not errors, e.g.,
-  # https://github.com/yihui/tinytex/issues/311 fix them and re-run engine
+  # https://github.com/rstudio/tinytex/issues/311 fix them and re-run engine
   if (install_packages && check_extra(logfile)) run_engine()
 
   # generate index
@@ -582,6 +582,9 @@ regex_errors = function() {
     colorprofiles.sty = c(
       '.* Package pdfx Error: No color profile ([^ ]+).*'
     ),
+    `lua-uni-algos.lua` = c(
+      ".* module '(lua-uni-normalize)' not found:.*"
+    ),
     tikz = c(
       # when a required tikz library is missing
       '.* (tikzlibrary[^ ]+?[.]code[.]tex).*'
@@ -590,7 +593,7 @@ regex_errors = function() {
       # missing .sty or commands
       ".* Loading '([^']+)' aborted!",
       ".*! LaTeX Error: File [`']([^']+)' not found.*",
-      ".* file ['`]?([^' ]+)'? not found.*",
+      ".* [fF]ile ['`]?([^' ]+)'? not found.*",
       '.*the language definition file ([^ ]+) .*',
       '.* \\(file ([^)]+)\\): cannot open .*',
       '.* open style file ([^ ]+).*',
@@ -610,7 +613,8 @@ detect_files = function(text) {
     v = grep_sub(p, '\\1', x)
     if (length(v) == 0) return(v)
     if (p == r$tikz && length(grep('! Package tikz Error:', text)) == 0) return()
-    for (i in c('epstopdf', 'colorprofiles.sty')) {
+    # these are some known filenames
+    for (i in c('epstopdf', grep('[.]', names(r), value = TRUE))) {
       if (p %in% r[[i]]) return(i)
     }
     if (p == r$fd) v = tolower(v)  # LGRcmr.fd -> lgrcmr.fd
@@ -667,7 +671,9 @@ fmtutil = function(usermode = FALSE, ...) {
 
 fc_cache = function(args = c('-v', '-r')) {
   tweak_path()
-  system2('fc-cache', args)
+  # run fc-cache on default dirs, then on the TinyTeX root dir
+  for (i in unique(c('', tinytex_root(error = FALSE))))
+    system2('fc-cache', shQuote(c(args, if (i != '') i)))
 }
 
 # refresh/update/regenerate everything
